@@ -4,10 +4,11 @@ require "mutex_m"
 require "minitest/parallel"
 
 ##
-# :include: README.txt
+# :include: README.rdoc
 
 module Minitest
-  VERSION = "5.4.2" # :nodoc:
+  VERSION = "5.8.0" # :nodoc:
+  ENCS = "".respond_to? :encoding # :nodoc:
 
   @@installed_at_exit ||= false
   @@after_run = []
@@ -19,7 +20,7 @@ module Minitest
   # Parallel test executor
 
   mc.send :attr_accessor, :parallel_executor
-  self.parallel_executor = Parallel::Executor.new((ENV['N'] || 2).to_i)
+  self.parallel_executor = Parallel::Executor.new((ENV["N"] || 2).to_i)
 
   ##
   # Filter object for backtraces.
@@ -106,7 +107,7 @@ module Minitest
   #           runnable.run(reporter, options)
   #             self.runnable_methods.each
   #               self.run_one_method(self, runnable_method, reporter)
-  #                 Minitest.run_one_method(klass, runnable_method, reporter)
+  #                 Minitest.run_one_method(klass, runnable_method)
   #                   klass.new(runnable_method).run
 
   def self.run args = []
@@ -122,8 +123,13 @@ module Minitest
     self.init_plugins options
     self.reporter = nil # runnables shouldn't depend on the reporter, ever
 
+    self.parallel_executor.start if parallel_executor.respond_to?(:start)
     reporter.start
-    __run reporter, options
+    begin
+      __run reporter, options
+    rescue Interrupt
+      warn "Interrupted. Exiting..."
+    end
     self.parallel_executor.shutdown
     reporter.report
 
@@ -165,7 +171,8 @@ module Minitest
         exit
       end
 
-      opts.on "-s", "--seed SEED", Integer, "Sets random seed" do |m|
+      desc = "Sets random seed. Also via env. Eg: SEED=n rake"
+      opts.on "-s", "--seed SEED", Integer, desc do |m|
         options[:seed] = m.to_i
       end
 
@@ -173,13 +180,13 @@ module Minitest
         options[:verbose] = true
       end
 
-      opts.on "-n", "--name PATTERN","Filter run on /pattern/ or string." do |a|
+      opts.on "-n", "--name PATTERN", "Filter run on /regexp/ or string." do |a|
         options[:filter] = a
       end
 
       unless extensions.empty?
         opts.separator ""
-        opts.separator "Known extensions: #{extensions.join(', ')}"
+        opts.separator "Known extensions: #{extensions.join(", ")}"
 
         extensions.each do |meth|
           msg = "plugin_#{meth}_options"
@@ -202,7 +209,7 @@ module Minitest
 
     unless options[:seed] then
       srand
-      options[:seed] = srand % 0xFFFF
+      options[:seed] = (ENV["SEED"] || srand).to_i % 0xFFFF
       orig_args << "--seed" << options[:seed].to_s
     end
 
@@ -275,13 +282,15 @@ module Minitest
     # reporter to record.
 
     def self.run reporter, options = {}
-      filter = options[:filter] || '/./'
-      filter = Regexp.new $1 if filter =~ /\/(.*)\//
+      filter = options[:filter] || "/./"
+      filter = Regexp.new $1 if filter =~ %r%/(.*)/%
 
       filtered_methods = self.runnable_methods.find_all { |m|
         filter === m || filter === "#{self}##{m}"
       }
 
+      return if filtered_methods.empty?
+
       with_info_handler reporter do
         filtered_methods.each do |method_name|
           run_one_method self, method_name, reporter
@@ -289,6 +298,12 @@ module Minitest
       end
     end
 
+    ##
+    # Runs a single method and has the reporter record the result.
+    # This was considered internal API but is factored out of run so
+    # that subclasses can specialize the running of an individual
+    # test. See Minitest::ParallelTest::ClassMethods for an example.
+
     def self.run_one_method klass, method_name, reporter
       reporter.record Minitest.run_one_method(klass, method_name)
     end
@@ -306,7 +321,7 @@ module Minitest
       on_signal "INFO", handler, &block
     end
 
-    SIGNALS = Signal.list
+    SIGNALS = Signal.list # :nodoc:
 
     def self.on_signal name, action # :nodoc:
       supported = SIGNALS[name]
@@ -494,7 +509,7 @@ module Minitest
     end
 
     def start # :nodoc:
-      self.start_time = Time.now
+      self.start_time = Minitest.clock_time
     end
 
     def record result # :nodoc:
@@ -508,7 +523,7 @@ module Minitest
       aggregate = results.group_by { |r| r.failure.class }
       aggregate.default = [] # dumb. group_by should provide this
 
-      self.total_time = Time.now - start_time
+      self.total_time = Minitest.clock_time - start_time
       self.failures   = aggregate[Assertion].size
       self.errors     = aggregate[UnexpectedError].size
       self.skips      = aggregate[Skip].size
@@ -563,9 +578,14 @@ module Minitest
       filtered_results = results.dup
       filtered_results.reject!(&:skipped?) unless options[:verbose]
 
-      filtered_results.each_with_index.map do |result, i|
+      s = filtered_results.each_with_index.map { |result, i|
         "\n%3d) %s" % [i+1, result]
-      end.join("\n") + "\n"
+      }.join("\n") + "\n"
+
+      s.force_encoding(io.external_encoding) if
+        ENCS and io.external_encoding and s.encoding != io.external_encoding
+
+      s
     end
 
     alias to_s aggregated_results
@@ -666,7 +686,7 @@ module Minitest
     attr_accessor :exception # :nodoc:
 
     def initialize exception # :nodoc:
-      super
+      super "Unexpected exception"
       self.exception = exception
     end
 
@@ -679,7 +699,7 @@ module Minitest
     end
 
     def message # :nodoc:
-      bt = Minitest::filter_backtrace(self.backtrace).join "\n    "
+      bt = Minitest.filter_backtrace(self.backtrace).join "\n    "
       "#{self.exception.class}: #{self.exception.message}\n    #{bt}"
     end
 
@@ -762,6 +782,16 @@ module Minitest
     raise "#{klass}#run _must_ return self" unless klass === result
     result
   end
+
+  if defined? Process::CLOCK_MONOTONIC
+    def self.clock_time
+      Process.clock_gettime Process::CLOCK_MONOTONIC
+    end
+  else
+    def self.clock_time
+      Time.now
+    end
+  end
 end
 
 require "minitest/test"
